# Author: Rick Hofstede, University of Twente <r.j.hofstede@utwente.nl>

#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-

# Measurement script for analyzing flow record expiration behavior (based on active timeout, idle timeout and TCP flags).
# More details: Section 4 of the following paper:
#       Measurement Artifacts in NetFlow Data
#       Rick Hofstede, Idilio Drago, Anna Sperotto, Ramin Sadre, Aiko Pras.
#       In: Proceedings of the Passive and Active Measurement conference (PAM 2013), 18-20 May 2013, Hong Kong, China

from scapy.all import *

src_host = '<src_ip>' # Enter the source IP address here
dst_host = '<dst_ip>' # Enter the destination address here
src_port = 26664
dst_port = 26664
idle_timeout = 30 # in seconds
active_timeout = 120 # in seconds
fast_aging = 0
fast_aging_timeout = 8 # in seconds
fast_aging_threshold = 32 # packets
tcp_flags_expiration = 0

p = IP(src = src_host, dst = dst_host)/TCP(sport = src_port, dport = dst_port)

# Measure whether (idle) flow record is expired after (exactly) i seconds
for i in range(idle_timeout - 2, idle_timeout + 20 + 1):
    p[TCP].sport = (src_port + i - idle_timeout + 2)
    p[TCP].dport = (dst_port + i - idle_timeout + 2)
    
    p[TCP].flags = 'S'
    send(p)
    
    if fast_aging == 1:
        p[TCP].flags = 'A'
        for j in range(0, fast_aging_threshold - 1):
            send(p)
    
    time.sleep(i)
    p[TCP].flags = 'P'
    send(p)

# Measure whether (active) flow record is expired after (exactly) i seconds
for i in range(active_timeout - 2, active_timeout + 20 + 1):
    start_time = int(time.time())
    p[TCP].sport = (src_port + 23 + i - active_timeout + 2)
    p[TCP].dport = (dst_port + 23 + i - active_timeout + 2)
    
    p[TCP].flags = 'S'
    send(p)
    
    p[TCP].flags = 'A'
    if fast_aging == 1:
        for j in range(0, fast_aging_threshold - 1):
            send(p)
    
    elapsed_time = int(time.time()) - start_time
    
    while (elapsed_time <= i):
        send(p)
        time.sleep(1)
        elapsed_time = int(time.time()) - start_time

if tcp_flags_expiration == 1:
    # Measure whether flow record that is expired by TCP FIN is immediately exported (delay of i seconds)
    for i in range(0, 20 + 1):
        p[TCP].sport = src_port + 46 + i
        p[TCP].dport = dst_port + 46 + i
        p[TCP].flags = 'S'
        send(p)
        
        p[TCP].flags = 'A'
        for j in range(0, 3):
    	    send(p)
        
        if fast_aging == 1:
    	    for j in range(0, fast_aging_threshold - 4):
                send(p)
        
        p[TCP].flags = 'F'
        send(p)
        time.sleep(i)
        
        p[TCP].flags = 'A'
        send(p)
